@@ -18,6 +18,7 @@ use core::{
 impl Add<UBig> for UBig {
     type Output = UBig;
 
+    #[inline]
     fn add(self, rhs: UBig) -> UBig {
         match (self.into_repr(), rhs.into_repr()) {
             (Small(word0), Small(word1)) => UBig::add_word(word0, word1),
@@ -37,16 +38,13 @@ impl Add<UBig> for UBig {
 impl Add<&UBig> for UBig {
     type Output = UBig;
 
+    #[inline]
     fn add(self, rhs: &UBig) -> UBig {
-        match self.into_repr() {
-            Small(word0) => match rhs.repr() {
-                Small(word1) => UBig::add_word(word0, *word1),
-                Large(buffer1) => UBig::add_large_word(buffer1.clone(), word0),
-            },
-            Large(buffer0) => match rhs.repr() {
-                Small(word1) => UBig::add_large_word(buffer0, *word1),
-                Large(buffer1) => UBig::add_large(buffer0, buffer1),
-            },
+        match (self.into_repr(), rhs.repr()) {
+            (Small(word0), Small(word1)) => UBig::add_word(word0, *word1),
+            (Small(word0), Large(buffer1)) => UBig::add_large_word(buffer1.clone(), word0),
+            (Large(buffer0), Small(word1)) => UBig::add_large_word(buffer0, *word1),
+            (Large(buffer0), Large(buffer1)) => UBig::add_large(buffer0, buffer1),
         }
     }
 }
@@ -54,6 +52,7 @@ impl Add<&UBig> for UBig {
 impl Add<UBig> for &UBig {
     type Output = UBig;
 
+    #[inline]
     fn add(self, rhs: UBig) -> UBig {
         rhs.add(self)
     }
@@ -62,6 +61,7 @@ impl Add<UBig> for &UBig {
 impl Add<&UBig> for &UBig {
     type Output = UBig;
 
+    #[inline]
     fn add(self, rhs: &UBig) -> UBig {
         match (self.repr(), rhs.repr()) {
             (Small(word0), Small(word1)) => UBig::add_word(*word0, *word1),
@@ -79,12 +79,14 @@ impl Add<&UBig> for &UBig {
 }
 
 impl AddAssign<UBig> for UBig {
+    #[inline]
     fn add_assign(&mut self, rhs: UBig) {
         *self = mem::take(self) + rhs;
     }
 }
 
 impl AddAssign<&UBig> for UBig {
+    #[inline]
     fn add_assign(&mut self, rhs: &UBig) {
         *self = mem::take(self) + rhs;
     }
@@ -93,42 +95,68 @@ impl AddAssign<&UBig> for UBig {
 impl Sub<UBig> for UBig {
     type Output = UBig;
 
+    #[inline]
     fn sub(self, rhs: UBig) -> UBig {
-        UBig::from_ibig_panic_on_overflow(IBig::sub_ubig_val_val(self, rhs))
+        match (self.into_repr(), rhs.into_repr()) {
+            (Small(word0), Small(word1)) => UBig::sub_word(word0, word1),
+            (Small(_), Large(_)) => UBig::panic_negative(),
+            (Large(buffer0), Small(word1)) => UBig::sub_large_word(buffer0, word1),
+            (Large(buffer0), Large(buffer1)) => UBig::sub_large(buffer0, &buffer1),
+        }
     }
 }
 
 impl Sub<&UBig> for UBig {
     type Output = UBig;
 
+    #[inline]
     fn sub(self, rhs: &UBig) -> UBig {
-        UBig::from_ibig_panic_on_overflow(IBig::sub_ubig_val_ref(self, rhs))
+        match (self.into_repr(), rhs.repr()) {
+            (Small(word0), Small(word1)) => UBig::sub_word(word0, *word1),
+            (Small(_), Large(_)) => UBig::panic_negative(),
+            (Large(buffer0), Small(word1)) => UBig::sub_large_word(buffer0, *word1),
+            (Large(buffer0), Large(buffer1)) => UBig::sub_large(buffer0, buffer1),
+        }
     }
 }
 
 impl Sub<UBig> for &UBig {
     type Output = UBig;
 
+    #[inline]
     fn sub(self, rhs: UBig) -> UBig {
-        UBig::from_ibig_panic_on_overflow(-IBig::sub_ubig_val_ref(rhs, self))
+        match (self.repr(), rhs.into_repr()) {
+            (Small(word0), Small(word1)) => UBig::sub_word(*word0, word1),
+            (Small(_), Large(_)) => UBig::panic_negative(),
+            (Large(buffer0), Small(word1)) => UBig::sub_large_word(buffer0.clone(), word1),
+            (Large(buffer0), Large(buffer1)) => UBig::sub_large_ref_val(buffer0, buffer1),
+        }
     }
 }
 
 impl Sub<&UBig> for &UBig {
     type Output = UBig;
 
+    #[inline]
     fn sub(self, rhs: &UBig) -> UBig {
-        UBig::from_ibig_panic_on_overflow(IBig::sub_ubig_ref_ref(self, rhs))
+        match (self.repr(), rhs.repr()) {
+            (Small(word0), Small(word1)) => UBig::sub_word(*word0, *word1),
+            (Small(_), Large(_)) => UBig::panic_negative(),
+            (Large(buffer0), Small(word1)) => UBig::sub_large_word(buffer0.clone(), *word1),
+            (Large(buffer0), Large(buffer1)) => UBig::sub_large(buffer0.clone(), buffer1),
+        }
     }
 }
 
 impl SubAssign<UBig> for UBig {
+    #[inline]
     fn sub_assign(&mut self, rhs: UBig) {
         *self = mem::take(self) - rhs;
     }
 }
 
 impl SubAssign<&UBig> for UBig {
+    #[inline]
     fn sub_assign(&mut self, rhs: &UBig) {
         *self = mem::take(self) - rhs;
     }
@@ -137,6 +165,7 @@ impl SubAssign<&UBig> for UBig {
 impl Add<IBig> for IBig {
     type Output = IBig;
 
+    #[inline]
     fn add(self, rhs: IBig) -> IBig {
         let (sign0, mag0) = self.into_sign_magnitude();
         let (sign1, mag1) = rhs.into_sign_magnitude();
@@ -152,6 +181,7 @@ impl Add<IBig> for IBig {
 impl Add<&IBig> for IBig {
     type Output = IBig;
 
+    #[inline]
     fn add(self, rhs: &IBig) -> IBig {
         let (sign0, mag0) = self.into_sign_magnitude();
         let (sign1, mag1) = (rhs.sign(), rhs.magnitude());
@@ -167,6 +197,7 @@ impl Add<&IBig> for IBig {
 impl Add<IBig> for &IBig {
     type Output = IBig;
 
+    #[inline]
     fn add(self, rhs: IBig) -> IBig {
         rhs.add(self)
     }
@@ -175,6 +206,7 @@ impl Add<IBig> for &IBig {
 impl Add<&IBig> for &IBig {
     type Output = IBig;
 
+    #[inline]
     fn add(self, rhs: &IBig) -> IBig {
         let (sign0, mag0) = (self.sign(), self.magnitude());
         let (sign1, mag1) = (rhs.sign(), rhs.magnitude());
@@ -188,12 +220,14 @@ impl Add<&IBig> for &IBig {
 }
 
 impl AddAssign<IBig> for IBig {
+    #[inline]
     fn add_assign(&mut self, rhs: IBig) {
         *self = mem::take(self) + rhs;
     }
 }
 
 impl AddAssign<&IBig> for IBig {
+    #[inline]
     fn add_assign(&mut self, rhs: &IBig) {
         *self = mem::take(self) + rhs;
     }
@@ -202,6 +236,7 @@ impl AddAssign<&IBig> for IBig {
 impl Sub<IBig> for IBig {
     type Output = IBig;
 
+    #[inline]
     fn sub(self, rhs: IBig) -> IBig {
         self + -rhs
     }
@@ -210,6 +245,7 @@ impl Sub<IBig> for IBig {
 impl Sub<&IBig> for IBig {
     type Output = IBig;
 
+    #[inline]
     fn sub(self, rhs: &IBig) -> IBig {
         -(-self + rhs)
     }
@@ -218,6 +254,7 @@ impl Sub<&IBig> for IBig {
 impl Sub<IBig> for &IBig {
     type Output = IBig;
 
+    #[inline]
     fn sub(self, rhs: IBig) -> IBig {
         self + -rhs
     }
@@ -226,6 +263,7 @@ impl Sub<IBig> for &IBig {
 impl Sub<&IBig> for &IBig {
     type Output = IBig;
 
+    #[inline]
     fn sub(self, rhs: &IBig) -> IBig {
         let (sign0, mag0) = (self.sign(), self.magnitude());
         let (sign1, mag1) = (rhs.sign(), rhs.magnitude());
@@ -239,12 +277,14 @@ impl Sub<&IBig> for &IBig {
 }
 
 impl SubAssign<IBig> for IBig {
+    #[inline]
     fn sub_assign(&mut self, rhs: IBig) {
         *self = mem::take(self) - rhs;
     }
 }
 
 impl SubAssign<&IBig> for IBig {
+    #[inline]
     fn sub_assign(&mut self, rhs: &IBig) {
         *self = mem::take(self) - rhs;
     }
@@ -255,6 +295,7 @@ macro_rules! impl_add_ubig_unsigned {
         impl Add<$t> for UBig {
             type Output = UBig;
 
+            #[inline]
             fn add(self, rhs: $t) -> UBig {
                 self.add_unsigned(rhs)
             }
@@ -263,6 +304,7 @@ macro_rules! impl_add_ubig_unsigned {
         impl Add<$t> for &UBig {
             type Output = UBig;
 
+            #[inline]
             fn add(self, rhs: $t) -> UBig {
                 self.add_ref_unsigned(rhs)
             }
@@ -272,6 +314,7 @@ macro_rules! impl_add_ubig_unsigned {
         helper_macros::forward_binop_swap_args!(impl Add<UBig> for $t, add);
 
         impl AddAssign<$t> for UBig {
+            #[inline]
             fn add_assign(&mut self, rhs: $t) {
                 self.add_assign_unsigned(rhs)
             }
@@ -282,6 +325,7 @@ macro_rules! impl_add_ubig_unsigned {
         impl Sub<$t> for UBig {
             type Output = UBig;
 
+            #[inline]
             fn sub(self, rhs: $t) -> UBig {
                 self.sub_unsigned(rhs)
             }
@@ -290,6 +334,7 @@ macro_rules! impl_add_ubig_unsigned {
         impl Sub<$t> for &UBig {
             type Output = UBig;
 
+            #[inline]
             fn sub(self, rhs: $t) -> UBig {
                 self.sub_ref_unsigned(rhs)
             }
@@ -298,6 +343,7 @@ macro_rules! impl_add_ubig_unsigned {
         helper_macros::forward_binop_second_arg_by_value!(impl Sub<$t> for UBig, sub);
 
         impl SubAssign<$t> for UBig {
+            #[inline]
             fn sub_assign(&mut self, rhs: $t) {
                 self.sub_assign_unsigned(rhs)
             }
@@ -319,6 +365,7 @@ macro_rules! impl_add_ubig_signed {
         impl Add<$t> for UBig {
             type Output = UBig;
 
+            #[inline]
             fn add(self, rhs: $t) -> UBig {
                 self.add_signed(rhs)
             }
@@ -327,6 +374,7 @@ macro_rules! impl_add_ubig_signed {
         impl Add<$t> for &UBig {
             type Output = UBig;
 
+            #[inline]
             fn add(self, rhs: $t) -> UBig {
                 self.add_ref_signed(rhs)
             }
@@ -336,6 +384,7 @@ macro_rules! impl_add_ubig_signed {
         helper_macros::forward_binop_swap_args!(impl Add<UBig> for $t, add);
 
         impl AddAssign<$t> for UBig {
+            #[inline]
             fn add_assign(&mut self, rhs: $t) {
                 self.add_assign_signed(rhs)
             }
@@ -346,13 +395,16 @@ macro_rules! impl_add_ubig_signed {
         impl Sub<$t> for UBig {
             type Output = UBig;
 
+            #[inline]
             fn sub(self, rhs: $t) -> UBig {
                 self.sub_signed(rhs)
             }
         }
+
         impl Sub<$t> for &UBig {
             type Output = UBig;
 
+            #[inline]
             fn sub(self, rhs: $t) -> UBig {
                 self.sub_ref_signed(rhs)
             }
@@ -361,6 +413,7 @@ macro_rules! impl_add_ubig_signed {
         helper_macros::forward_binop_second_arg_by_value!(impl Sub<$t> for UBig, sub);
 
         impl SubAssign<$t> for UBig {
+            #[inline]
             fn sub_assign(&mut self, rhs: $t) {
                 self.sub_assign_signed(rhs)
             }
@@ -382,6 +435,7 @@ macro_rules! impl_add_ibig_primitive {
         impl Add<$t> for IBig {
             type Output = IBig;
 
+            #[inline]
             fn add(self, rhs: $t) -> IBig {
                 self.add_primitive(rhs)
             }
@@ -390,6 +444,7 @@ macro_rules! impl_add_ibig_primitive {
         impl Add<$t> for &IBig {
             type Output = IBig;
 
+            #[inline]
             fn add(self, rhs: $t) -> IBig {
                 self.add_ref_primitive(rhs)
             }
@@ -399,6 +454,7 @@ macro_rules! impl_add_ibig_primitive {
         helper_macros::forward_binop_swap_args!(impl Add<IBig> for $t, add);
 
         impl AddAssign<$t> for IBig {
+            #[inline]
             fn add_assign(&mut self, rhs: $t) {
                 self.add_assign_primitive(rhs)
             }
@@ -409,6 +465,7 @@ macro_rules! impl_add_ibig_primitive {
         impl Sub<$t> for IBig {
             type Output = IBig;
 
+            #[inline]
             fn sub(self, rhs: $t) -> IBig {
                 self.sub_primitive(rhs)
             }
@@ -417,6 +474,7 @@ macro_rules! impl_add_ibig_primitive {
         impl Sub<$t> for &IBig {
             type Output = IBig;
 
+            #[inline]
             fn sub(self, rhs: $t) -> IBig {
                 self.sub_ref_primitive(rhs)
             }
@@ -425,6 +483,7 @@ macro_rules! impl_add_ibig_primitive {
         impl Sub<IBig> for $t {
             type Output = IBig;
 
+            #[inline]
             fn sub(self, rhs: IBig) -> IBig {
                 rhs.sub_from_primitive(self)
             }
@@ -433,6 +492,7 @@ macro_rules! impl_add_ibig_primitive {
         impl Sub<&IBig> for $t {
             type Output = IBig;
 
+            #[inline]
             fn sub(self, rhs: &IBig) -> IBig {
                 rhs.sub_ref_from_primitive(self)
             }
@@ -442,6 +502,7 @@ macro_rules! impl_add_ibig_primitive {
         helper_macros::forward_binop_first_arg_by_value!(impl Sub<IBig> for $t, sub);
 
         impl SubAssign<$t> for IBig {
+            #[inline]
             fn sub_assign(&mut self, rhs: $t) {
                 self.sub_assign_primitive(rhs)
             }
@@ -466,6 +527,7 @@ impl_add_ibig_primitive!(isize);
 
 impl UBig {
     /// Add two `Word`s.
+    #[inline]
     fn add_word(a: Word, b: Word) -> UBig {
         let (res, overflow) = a.overflowing_add(b);
         if overflow {
@@ -501,62 +563,105 @@ impl UBig {
         buffer.into()
     }
 
+    /// Subtract two `Word`s.
+    #[inline]
+    fn sub_word(a: Word, b: Word) -> UBig {
+        match a.checked_sub(b) {
+            Some(res) => UBig::from_word(res),
+            None => UBig::panic_negative(),
+        }
+    }
+
     fn sub_large_word(mut lhs: Buffer, rhs: Word) -> UBig {
         let overflow = add::sub_word_in_place(&mut lhs, rhs);
         assert!(!overflow);
         lhs.into()
     }
 
+    fn sub_large(mut lhs: Buffer, rhs: &[Word]) -> UBig {
+        if lhs.len() < rhs.len() || add::sub_in_place(&mut lhs, rhs) {
+            UBig::panic_negative();
+        }
+        lhs.into()
+    }
+
+    fn sub_large_ref_val(lhs: &[Word], mut rhs: Buffer) -> UBig {
+        let n = rhs.len();
+        if lhs.len() < n {
+            UBig::panic_negative();
+        }
+        let borrow = add::sub_same_len_in_place_swap(&lhs[..n], &mut rhs);
+        rhs.ensure_capacity(lhs.len());
+        rhs.extend(&lhs[n..]);
+        if borrow && add::sub_one_in_place(&mut rhs[n..]) {
+            UBig::panic_negative();
+        }
+        rhs.into()
+    }
+
+    #[inline]
     fn add_unsigned<T: PrimitiveUnsigned>(self, rhs: T) -> UBig {
         self + UBig::from_unsigned(rhs)
     }
 
+    #[inline]
     fn add_ref_unsigned<T: PrimitiveUnsigned>(&self, rhs: T) -> UBig {
         self + UBig::from_unsigned(rhs)
     }
 
+    #[inline]
     fn add_assign_unsigned<T: PrimitiveUnsigned>(&mut self, rhs: T) {
         *self += UBig::from_unsigned(rhs)
     }
 
+    #[inline]
     fn sub_unsigned<T: PrimitiveUnsigned>(self, rhs: T) -> UBig {
         self - UBig::from_unsigned(rhs)
     }
 
+    #[inline]
     fn sub_ref_unsigned<T: PrimitiveUnsigned>(&self, rhs: T) -> UBig {
         self - UBig::from_unsigned(rhs)
     }
 
+    #[inline]
     fn sub_assign_unsigned<T: PrimitiveUnsigned>(&mut self, rhs: T) {
         *self -= UBig::from_unsigned(rhs)
     }
 
+    #[inline]
     fn add_signed<T: PrimitiveSigned>(self, rhs: T) -> UBig {
         UBig::from_ibig_panic_on_overflow(IBig::from(self) + IBig::from_signed(rhs))
     }
 
+    #[inline]
     fn add_ref_signed<T: PrimitiveSigned>(&self, rhs: T) -> UBig {
         UBig::from_ibig_panic_on_overflow(IBig::from(self) + IBig::from_signed(rhs))
     }
 
+    #[inline]
     fn add_assign_signed<T: PrimitiveSigned>(&mut self, rhs: T) {
         *self = mem::take(self).add_signed(rhs)
     }
 
+    #[inline]
     fn sub_signed<T: PrimitiveSigned>(self, rhs: T) -> UBig {
         UBig::from_ibig_panic_on_overflow(IBig::from(self) - IBig::from_signed(rhs))
     }
 
+    #[inline]
     fn sub_ref_signed<T: PrimitiveSigned>(&self, rhs: T) -> UBig {
         UBig::from_ibig_panic_on_overflow(IBig::from(self) - IBig::from_signed(rhs))
     }
 
+    #[inline]
     fn sub_assign_signed<T: PrimitiveSigned>(&mut self, rhs: T) {
         *self = mem::take(self).sub_signed(rhs)
     }
 }
 
 impl IBig {
+    #[inline]
     fn sub_ubig_val_val(lhs: UBig, rhs: UBig) -> IBig {
         match (lhs.into_repr(), rhs.into_repr()) {
             (Small(word0), Small(word1)) => IBig::sub_word_word(word0, word1),
@@ -572,19 +677,17 @@ impl IBig {
         }
     }
 
+    #[inline]
     fn sub_ubig_val_ref(lhs: UBig, rhs: &UBig) -> IBig {
-        match lhs.into_repr() {
-            Small(word0) => match rhs.repr() {
-                Small(word1) => IBig::sub_word_word(word0, *word1),
-                Large(buffer1) => -IBig::sub_large_word(buffer1.clone(), word0),
-            },
-            Large(buffer0) => match rhs.repr() {
-                Small(word1) => IBig::sub_large_word(buffer0, *word1),
-                Large(buffer1) => IBig::sub_large(buffer0, buffer1),
-            },
+        match (lhs.into_repr(), rhs.repr()) {
+            (Small(word0), Small(word1)) => IBig::sub_word_word(word0, *word1),
+            (Small(word0), Large(buffer1)) => -IBig::sub_large_word(buffer1.clone(), word0),
+            (Large(buffer0), Small(word1)) => IBig::sub_large_word(buffer0, *word1),
+            (Large(buffer0), Large(buffer1)) => IBig::sub_large(buffer0, buffer1),
         }
     }
 
+    #[inline]
     fn sub_ubig_ref_ref(lhs: &UBig, rhs: &UBig) -> IBig {
         match (lhs.repr(), rhs.repr()) {
             (Small(word0), Small(word1)) => IBig::sub_word_word(*word0, *word1),
@@ -600,11 +703,13 @@ impl IBig {
         }
     }
 
+    #[inline]
     fn sub_word_word(lhs: Word, rhs: Word) -> IBig {
-        if lhs >= rhs {
-            IBig::from(lhs - rhs)
+        let (val, overflow) = lhs.overflowing_sub(rhs);
+        if !overflow {
+            IBig::from(val)
         } else {
-            -IBig::from(rhs - lhs)
+            -IBig::from(val.wrapping_neg())
         }
     }
 
@@ -617,18 +722,12 @@ impl IBig {
             let sign = add::sub_in_place_with_sign(&mut lhs, rhs);
             IBig::from_sign_magnitude(sign, lhs.into())
         } else {
-            let n = lhs.len();
-            let borrow = add::sub_same_len_in_place_swap(&rhs[..n], &mut lhs);
-            lhs.ensure_capacity(rhs.len());
-            lhs.extend(&rhs[n..]);
-            if borrow {
-                let overflow = add::sub_one_in_place(&mut lhs[n..]);
-                assert!(!overflow);
-            }
-            IBig::from_sign_magnitude(Negative, lhs.into())
+            let res = UBig::sub_large_ref_val(rhs, lhs);
+            IBig::from_sign_magnitude(Negative, res)
         }
     }
 
+    #[inline]
     fn add_primitive<T>(self, rhs: T) -> IBig
     where
         IBig: From<T>,
@@ -636,6 +735,7 @@ impl IBig {
         self + IBig::from(rhs)
     }
 
+    #[inline]
     fn add_ref_primitive<T>(&self, rhs: T) -> IBig
     where
         IBig: From<T>,
@@ -643,6 +743,7 @@ impl IBig {
         self + IBig::from(rhs)
     }
 
+    #[inline]
     fn add_assign_primitive<T>(&mut self, rhs: T)
     where
         IBig: From<T>,
@@ -650,6 +751,7 @@ impl IBig {
         *self += IBig::from(rhs)
     }
 
+    #[inline]
     fn sub_primitive<T>(self, rhs: T) -> IBig
     where
         IBig: From<T>,
@@ -657,6 +759,7 @@ impl IBig {
         self - IBig::from(rhs)
     }
 
+    #[inline]
     fn sub_ref_primitive<T>(&self, rhs: T) -> IBig
     where
         IBig: From<T>,
@@ -664,6 +767,7 @@ impl IBig {
         self - IBig::from(rhs)
     }
 
+    #[inline]
     fn sub_assign_primitive<T>(&mut self, rhs: T)
     where
         IBig: From<T>,
@@ -671,6 +775,7 @@ impl IBig {
         *self -= IBig::from(rhs)
     }
 
+    #[inline]
     fn sub_from_primitive<T>(self, rhs: T) -> IBig
     where
         IBig: From<T>,
@@ -678,6 +783,7 @@ impl IBig {
         IBig::from(rhs) - self
     }
 
+    #[inline]
     fn sub_ref_from_primitive<T>(&self, rhs: T) -> IBig
     where
         IBig: From<T>,
