@@ -51,6 +51,13 @@
 //! assert_eq!(format!("{}", x - y), "6902 (mod 10000)");
 //! # Ok::<(), ParseError>(())
 //! ```
+//!
+//! # Optional dependencies
+//!
+//! * `std` (default): for `std::error::Error`.
+//! * `num-traits` (default): integral traits.
+//! * `rand` (default): random number generation.
+//! * `serde`: serialization and deserialization.
 
 #![cfg_attr(not(feature = "std"), no_std)]
 
@@ -61,6 +68,7 @@ pub use crate::{ibig::IBig, ubig::UBig};
 mod add;
 mod add_ops;
 mod arch;
+mod assert;
 mod bits;
 mod buffer;
 mod cmp;
@@ -93,3 +101,6 @@ pub mod rand;
 
 #[cfg(feature = "num-traits")]
 mod num_traits;
+
+#[cfg(feature = "serde")]
+mod serde;
