@@ -15,10 +15,12 @@ use core::{
     mem,
     ops::{Mul, MulAssign},
 };
+use static_assertions::const_assert;
 
 impl Mul<UBig> for UBig {
     type Output = UBig;
 
+    #[inline]
     fn mul(self, rhs: UBig) -> UBig {
         match (self.into_repr(), rhs.into_repr()) {
             (Small(word0), Small(word1)) => UBig::mul_word(word0, word1),
@@ -32,16 +34,13 @@ impl Mul<UBig> for UBig {
 impl Mul<&UBig> for UBig {
     type Output = UBig;
 
+    #[inline]
     fn mul(self, rhs: &UBig) -> UBig {
-        match self.into_repr() {
-            Small(word0) => match rhs.repr() {
-                Small(word1) => UBig::mul_word(word0, *word1),
-                Large(buffer1) => UBig::mul_large_word(buffer1.clone(), word0),
-            },
-            Large(buffer0) => match rhs.repr() {
-                Small(word1) => UBig::mul_large_word(buffer0, *word1),
-                Large(buffer1) => UBig::mul_large(&buffer0, buffer1),
-            },
+        match (self.into_repr(), rhs.repr()) {
+            (Small(word0), Small(word1)) => UBig::mul_word(word0, *word1),
+            (Small(word0), Large(buffer1)) => UBig::mul_large_word(buffer1.clone(), word0),
+            (Large(buffer0), Small(word1)) => UBig::mul_large_word(buffer0, *word1),
+            (Large(buffer0), Large(buffer1)) => UBig::mul_large(&buffer0, buffer1),
         }
     }
 }
@@ -49,6 +48,7 @@ impl Mul<&UBig> for UBig {
 impl Mul<UBig> for &UBig {
     type Output = UBig;
 
+    #[inline]
     fn mul(self, rhs: UBig) -> UBig {
         rhs.mul(self)
     }
@@ -57,6 +57,7 @@ impl Mul<UBig> for &UBig {
 impl Mul<&UBig> for &UBig {
     type Output = UBig;
 
+    #[inline]
     fn mul(self, rhs: &UBig) -> UBig {
         match (self.repr(), rhs.repr()) {
             (Small(word0), Small(word1)) => UBig::mul_word(*word0, *word1),
@@ -68,12 +69,14 @@ impl Mul<&UBig> for &UBig {
 }
 
 impl MulAssign<UBig> for UBig {
+    #[inline]
     fn mul_assign(&mut self, rhs: UBig) {
         *self = mem::take(self) * rhs;
     }
 }
 
 impl MulAssign<&UBig> for UBig {
+    #[inline]
     fn mul_assign(&mut self, rhs: &UBig) {
         *self = mem::take(self) * rhs;
     }
@@ -82,6 +85,7 @@ impl MulAssign<&UBig> for UBig {
 impl Mul<IBig> for IBig {
     type Output = IBig;
 
+    #[inline]
     fn mul(self, rhs: IBig) -> IBig {
         let (sign0, mag0) = self.into_sign_magnitude();
         let (sign1, mag1) = rhs.into_sign_magnitude();
@@ -92,6 +96,7 @@ impl Mul<IBig> for IBig {
 impl Mul<&IBig> for IBig {
     type Output = IBig;
 
+    #[inline]
     fn mul(self, rhs: &IBig) -> IBig {
         let (sign0, mag0) = self.into_sign_magnitude();
         let (sign1, mag1) = (rhs.sign(), rhs.magnitude());
@@ -102,6 +107,7 @@ impl Mul<&IBig> for IBig {
 impl Mul<IBig> for &IBig {
     type Output = IBig;
 
+    #[inline]
     fn mul(self, rhs: IBig) -> IBig {
         rhs.mul(self)
     }
@@ -110,6 +116,7 @@ impl Mul<IBig> for &IBig {
 impl Mul<&IBig> for &IBig {
     type Output = IBig;
 
+    #[inline]
     fn mul(self, rhs: &IBig) -> IBig {
         let (sign0, mag0) = (self.sign(), self.magnitude());
         let (sign1, mag1) = (rhs.sign(), rhs.magnitude());
@@ -118,12 +125,14 @@ impl Mul<&IBig> for &IBig {
 }
 
 impl MulAssign<IBig> for IBig {
+    #[inline]
     fn mul_assign(&mut self, rhs: IBig) {
         *self = mem::take(self) * rhs;
     }
 }
 
 impl MulAssign<&IBig> for IBig {
+    #[inline]
     fn mul_assign(&mut self, rhs: &IBig) {
         *self = mem::take(self) * rhs;
     }
@@ -132,6 +141,7 @@ impl MulAssign<&IBig> for IBig {
 impl Mul<Sign> for Sign {
     type Output = Sign;
 
+    #[inline]
     fn mul(self, rhs: Sign) -> Sign {
         match (self, rhs) {
             (Positive, Positive) => Positive,
@@ -143,6 +153,7 @@ impl Mul<Sign> for Sign {
 }
 
 impl MulAssign<Sign> for Sign {
+    #[inline]
     fn mul_assign(&mut self, rhs: Sign) {
         *self = *self * rhs;
     }
@@ -153,6 +164,7 @@ macro_rules! impl_mul_ubig_unsigned {
         impl Mul<$t> for UBig {
             type Output = UBig;
 
+            #[inline]
             fn mul(self, rhs: $t) -> UBig {
                 self.mul_unsigned(rhs)
             }
@@ -161,6 +173,7 @@ macro_rules! impl_mul_ubig_unsigned {
         impl Mul<$t> for &UBig {
             type Output = UBig;
 
+            #[inline]
             fn mul(self, rhs: $t) -> UBig {
                 self.mul_ref_unsigned(rhs)
             }
@@ -170,6 +183,7 @@ macro_rules! impl_mul_ubig_unsigned {
         helper_macros::forward_binop_swap_args!(impl Mul<UBig> for $t, mul);
 
         impl MulAssign<$t> for UBig {
+            #[inline]
             fn mul_assign(&mut self, rhs: $t) {
                 self.mul_assign_unsigned(rhs)
             }
@@ -191,6 +205,7 @@ macro_rules! impl_mul_ubig_signed {
         impl Mul<$t> for UBig {
             type Output = UBig;
 
+            #[inline]
             fn mul(self, rhs: $t) -> UBig {
                 self.mul_signed(rhs)
             }
@@ -199,6 +214,7 @@ macro_rules! impl_mul_ubig_signed {
         impl Mul<$t> for &UBig {
             type Output = UBig;
 
+            #[inline]
             fn mul(self, rhs: $t) -> UBig {
                 self.mul_ref_signed(rhs)
             }
@@ -208,6 +224,7 @@ macro_rules! impl_mul_ubig_signed {
         helper_macros::forward_binop_swap_args!(impl Mul<UBig> for $t, mul);
 
         impl MulAssign<$t> for UBig {
+            #[inline]
             fn mul_assign(&mut self, rhs: $t) {
                 self.mul_assign_signed(rhs)
             }
@@ -229,6 +246,7 @@ macro_rules! impl_mul_ibig_primitive {
         impl Mul<$t> for IBig {
             type Output = IBig;
 
+            #[inline]
             fn mul(self, rhs: $t) -> IBig {
                 self.mul_primitive(rhs)
             }
@@ -237,6 +255,7 @@ macro_rules! impl_mul_ibig_primitive {
         impl Mul<$t> for &IBig {
             type Output = IBig;
 
+            #[inline]
             fn mul(self, rhs: $t) -> IBig {
                 self.mul_ref_primitive(rhs)
             }
@@ -246,6 +265,7 @@ macro_rules! impl_mul_ibig_primitive {
         helper_macros::forward_binop_swap_args!(impl Mul<IBig> for $t, mul);
 
         impl MulAssign<$t> for IBig {
+            #[inline]
             fn mul_assign(&mut self, rhs: $t) {
                 self.mul_assign_primitive(rhs)
             }
@@ -270,6 +290,7 @@ impl_mul_ibig_primitive!(isize);
 
 impl UBig {
     /// Multiply two `Word`s.
+    #[inline]
     fn mul_word(a: Word, b: Word) -> UBig {
         UBig::from(extend_word(a) * extend_word(b))
     }
@@ -293,6 +314,8 @@ impl UBig {
     fn mul_large(lhs: &[Word], rhs: &[Word]) -> UBig {
         debug_assert!(lhs.len() >= 2 && rhs.len() >= 2);
 
+        // This may be 1 too large.
+        const_assert!(Buffer::MAX_CAPACITY - UBig::MAX_LEN >= 1);
         let res_len = lhs.len() + rhs.len();
         let mut buffer = Buffer::allocate(res_len);
         buffer.push_zeros(res_len);
@@ -307,32 +330,39 @@ impl UBig {
         buffer.into()
     }
 
+    #[inline]
     fn mul_unsigned<T: PrimitiveUnsigned>(self, rhs: T) -> UBig {
         self * UBig::from_unsigned(rhs)
     }
 
+    #[inline]
     fn mul_ref_unsigned<T: PrimitiveUnsigned>(&self, rhs: T) -> UBig {
         self * UBig::from_unsigned(rhs)
     }
 
+    #[inline]
     fn mul_assign_unsigned<T: PrimitiveUnsigned>(&mut self, rhs: T) {
         *self *= UBig::from_unsigned(rhs)
     }
 
+    #[inline]
     fn mul_signed<T: PrimitiveSigned>(self, rhs: T) -> UBig {
         UBig::from_ibig_panic_on_overflow(IBig::from(self) * IBig::from_signed(rhs))
     }
 
+    #[inline]
     fn mul_ref_signed<T: PrimitiveSigned>(&self, rhs: T) -> UBig {
         UBig::from_ibig_panic_on_overflow(IBig::from(self) * IBig::from_signed(rhs))
     }
 
+    #[inline]
     fn mul_assign_signed<T: PrimitiveSigned>(&mut self, rhs: T) {
         *self = mem::take(self).mul_signed(rhs)
     }
 }
 
 impl IBig {
+    #[inline]
     fn mul_primitive<T>(self, rhs: T) -> IBig
     where
         IBig: From<T>,
@@ -340,6 +370,7 @@ impl IBig {
         self * IBig::from(rhs)
     }
 
+    #[inline]
     fn mul_ref_primitive<T>(&self, rhs: T) -> IBig
     where
         IBig: From<T>,
@@ -347,6 +378,7 @@ impl IBig {
         self * IBig::from(rhs)
     }
 
+    #[inline]
     fn mul_assign_primitive<T>(&mut self, rhs: T)
     where
         IBig: From<T>,
