@@ -14,11 +14,13 @@ pub(crate) type Digit = u32;
 pub(crate) const MAX_RADIX: Digit = 36;
 
 /// Is a radix in valid range?
+#[inline]
 pub(crate) fn is_radix_valid(radix: Digit) -> bool {
     (2..=MAX_RADIX).contains(&radix)
 }
 
 /// Panics if `radix` is not in valid range.
+#[inline]
 pub(crate) fn check_radix_valid(radix: Digit) {
     if !is_radix_valid(radix) {
         panic!("Invalid radix: {}", radix);
@@ -52,7 +54,7 @@ pub(crate) fn digit_from_utf8_byte(byte: u8, radix: Digit) -> Option<Digit> {
 }
 
 /// Maximum number of digits that a `Word` can ever have for any non-power-of-2 radix.
-pub(crate) const MAX_WORD_DIGITS_NON_POW_2: usize = RADIX_INFO_TABLE[3].digits_per_word + 1;
+pub(crate) const MAX_WORD_DIGITS_NON_POW_2: usize = RadixInfo::for_radix(3).digits_per_word + 1;
 
 /// Properties of a given radix.
 #[derive(Clone, Copy)]
@@ -73,6 +75,7 @@ pub(crate) struct RadixInfo {
 }
 
 /// RadixInfo for a given radix.
+#[inline]
 pub(crate) fn radix_info(radix: Digit) -> &'static RadixInfo {
     debug_assert!(is_radix_valid(radix));
     &RADIX_INFO_TABLE[radix as usize]
@@ -109,7 +112,7 @@ impl RadixInfo {
 
 type RadixInfoTable = [RadixInfo; MAX_RADIX as usize + 1];
 
-const RADIX_INFO_TABLE: &RadixInfoTable = &generate_radix_info_table();
+static RADIX_INFO_TABLE: RadixInfoTable = generate_radix_info_table();
 
 const fn generate_radix_info_table() -> RadixInfoTable {
     let mut table = [RadixInfo {
